@@ -1,80 +1,102 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * E-table-specification.c: Implements a savable description of the inital state of a table.
+ * e-table-specification.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
  * Authors:
  *   Chris Lahey <clahey@ximian.com>
- *   Miguel de Icaza (miguel@ximian.com)
  *
- * (C) 2000, 2001 Ximian, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
 #include "e-table-specification.h"
 
 #include <stdlib.h>
-#include <gtk/gtksignal.h>
-#include <gnome-xml/parser.h>
-#include <gnome-xml/xmlmemory.h>
+#include <string.h>
+
+#include <libxml/parser.h>
+#include <libxml/xmlmemory.h>
 #include "gal/util/e-util.h"
 #include "gal/util/e-xml-utils.h"
 
-#define PARENT_TYPE (gtk_object_get_type ())
-
-static GtkObjectClass *etsp_parent_class;
+static GObjectClass *etsp_parent_class;
 
 static void
-etsp_destroy (GtkObject *object)
+etsp_finalize (GObject *object)
 {
 	ETableSpecification *etsp = E_TABLE_SPECIFICATION (object);
 	int i;
 
 	if (etsp->columns) {
 		for (i = 0; etsp->columns[i]; i++) {
-			gtk_object_unref (GTK_OBJECT (etsp->columns[i]));
+			g_object_unref (etsp->columns[i]);
 		}
 		g_free (etsp->columns);
+		etsp->columns = NULL;
 	}
 
 	if (etsp->state)
-		gtk_object_unref (GTK_OBJECT (etsp->state));
-	g_free (etsp->click_to_add_message);
-
-	etsp->columns              = NULL;
+		g_object_unref (etsp->state);
 	etsp->state                = NULL;
+
+	g_free (etsp->click_to_add_message);
 	etsp->click_to_add_message = NULL;
 
-	GTK_OBJECT_CLASS (etsp_parent_class)->destroy (object);
+	g_free (etsp->domain);
+	etsp->domain		   = NULL;
+
+	etsp_parent_class->finalize (object);
 }
 
 static void
-etsp_class_init (GtkObjectClass *klass)
+etsp_class_init (GObjectClass *klass)
 {
-	etsp_parent_class = gtk_type_class (PARENT_TYPE);
+	etsp_parent_class = g_type_class_peek_parent (klass);
 	
-	klass->destroy = etsp_destroy;
+	klass->finalize = etsp_finalize;
 }
 
 static void
 etsp_init (ETableSpecification *etsp)
 {
-	etsp->columns               = NULL;
-	etsp->state                 = NULL;
+	etsp->columns                = NULL;
+	etsp->state                  = NULL;
 
-	etsp->no_headers            = FALSE;
-	etsp->click_to_add          = FALSE;
-	etsp->horizontal_draw_grid  = FALSE;
-	etsp->vertical_draw_grid    = FALSE;
-	etsp->draw_focus            = TRUE;
-	etsp->horizontal_scrolling  = FALSE;
+	etsp->alternating_row_colors = TRUE;
+	etsp->no_headers             = FALSE;
+	etsp->click_to_add           = FALSE;
+	etsp->click_to_add_end       = FALSE;
+	etsp->horizontal_draw_grid   = FALSE;
+	etsp->vertical_draw_grid     = FALSE;
+	etsp->draw_focus             = TRUE;
+	etsp->horizontal_scrolling   = FALSE;
+	etsp->horizontal_resize      = FALSE;
+	etsp->allow_grouping         = TRUE;
 
-	etsp->cursor_mode           = E_CURSOR_SIMPLE;
-	etsp->selection_mode        = GTK_SELECTION_MULTIPLE;
+	etsp->cursor_mode            = E_CURSOR_SIMPLE;
+	etsp->selection_mode         = GTK_SELECTION_MULTIPLE;
 
-	etsp->click_to_add_message  = NULL;
+	etsp->click_to_add_message   = NULL;
+	etsp->domain                 = NULL;
 }
 
-E_MAKE_TYPE (e_table_specification, "ETableSpecification", ETableSpecification, etsp_class_init, etsp_init, PARENT_TYPE);
+E_MAKE_TYPE (e_table_specification, "ETableSpecification", ETableSpecification, etsp_class_init, etsp_init, G_TYPE_OBJECT)
 
 /**
  * e_table_specification_new:
@@ -87,7 +109,7 @@ E_MAKE_TYPE (e_table_specification, "ETableSpecification", ETableSpecification,
 ETableSpecification *
 e_table_specification_new (void)
 {
-	ETableSpecification *etsp = gtk_type_new (E_TABLE_SPECIFICATION_TYPE);
+	ETableSpecification *etsp = g_object_new (E_TABLE_SPECIFICATION_TYPE, NULL);
 
 	return (ETableSpecification *) etsp;
 }
@@ -163,6 +185,8 @@ e_table_specification_load_from_node (ETableSpecification *specification,
 
 	specification->no_headers = e_xml_get_bool_prop_by_name (node, "no-headers");
 	specification->click_to_add = e_xml_get_bool_prop_by_name (node, "click-to-add");
+	specification->click_to_add_end = e_xml_get_bool_prop_by_name (node, "click-to-add-end") && specification->click_to_add;
+	specification->alternating_row_colors = e_xml_get_bool_prop_by_name_with_default (node, "alternating-row-colors", TRUE);
 	specification->horizontal_draw_grid = e_xml_get_bool_prop_by_name (node, "horizontal-draw-grid");
 	specification->vertical_draw_grid = e_xml_get_bool_prop_by_name (node, "vertical-draw-grid");
 	if (e_xml_get_bool_prop_by_name_with_default(node, "draw-grid", TRUE) ==
@@ -172,6 +196,8 @@ e_table_specification_load_from_node (ETableSpecification *specification,
 	}
 	specification->draw_focus = e_xml_get_bool_prop_by_name_with_default (node, "draw-focus", TRUE);
 	specification->horizontal_scrolling = e_xml_get_bool_prop_by_name_with_default (node, "horizontal-scrolling", FALSE);
+	specification->horizontal_resize = e_xml_get_bool_prop_by_name_with_default (node, "horizontal-resize", FALSE);
+	specification->allow_grouping = e_xml_get_bool_prop_by_name_with_default (node, "allow-grouping", TRUE);
 
 	specification->selection_mode = GTK_SELECTION_MULTIPLE;
 	temp = e_xml_get_string_prop_by_name (node, "selection-mode");
@@ -192,18 +218,27 @@ e_table_specification_load_from_node (ETableSpecification *specification,
 		specification->cursor_mode = E_CURSOR_SPREADSHEET;
 	}
 	g_free (temp);
-	g_free (specification->click_to_add_message);
 
+	g_free (specification->click_to_add_message);
 	specification->click_to_add_message =
 		e_xml_get_string_prop_by_name (
 			node, "_click-to-add-message");
 
+	g_free (specification->domain);
+	specification->domain =
+		e_xml_get_string_prop_by_name (
+			node, "gettext-domain");
+	if (specification->domain && !*specification->domain) {
+		g_free (specification->domain);
+		specification->domain = NULL;
+	}
+
 	if (specification->state)
-		gtk_object_unref (GTK_OBJECT (specification->state));
+		g_object_unref (specification->state);
 	specification->state = NULL;
 	if (specification->columns) {
 		for (i = 0; specification->columns[i]; i++) {
-			gtk_object_unref (GTK_OBJECT (specification->columns[i]));
+			g_object_unref (specification->columns[i]);
 		}
 		g_free (specification->columns);
 	}
@@ -213,14 +248,21 @@ e_table_specification_load_from_node (ETableSpecification *specification,
 		if (!strcmp (children->name, "ETableColumn")) {
 			ETableColumnSpecification *col_spec = e_table_column_specification_new ();
 
+			g_object_ref (col_spec);
 			e_table_column_specification_load_from_node (col_spec, children);
 			list = g_list_append (list, col_spec);
 		} else if (specification->state == NULL && !strcmp (children->name, "ETableState")) {
 			specification->state = e_table_state_new ();
 			e_table_state_load_from_node (specification->state, children);
+			e_table_sort_info_set_can_group (specification->state->sort_info, specification->allow_grouping);
 		}
 	}
 
+	if (specification->state == NULL) {
+		/* Make the default state.  */
+		specification->state = e_table_state_vanilla (g_list_length (list));
+	}
+
 	specification->columns = g_new (ETableColumnSpecification *, g_list_length (list) + 1);
 	for (list2 = list, i = 0; list2; list2 = g_list_next (list2), i++) {
 		specification->columns[i] = list2->data;
@@ -236,21 +278,29 @@ e_table_specification_load_from_node (ETableSpecification *specification,
  *
  * This routine stores the @specification into @filename.  
  *
- * Returns: the number of bytes written or -1 on error.
+ * Returns: 0 on success or -1 on error.
  */
 int
 e_table_specification_save_to_file (ETableSpecification *specification,
 				    const char          *filename)
 {
 	xmlDoc *doc;
-
+	int ret;
+	
 	g_return_val_if_fail (specification != NULL, -1);
 	g_return_val_if_fail (filename != NULL, -1);
 	g_return_val_if_fail (E_IS_TABLE_SPECIFICATION (specification), -1);
 	
-	doc = xmlNewDoc ("1.0");
+	if ((doc = xmlNewDoc ("1.0")) == NULL)
+		return -1;
+	
 	xmlDocSetRootElement (doc, e_table_specification_save_to_node (specification, doc));
-	return xmlSaveFile (filename, doc);
+	
+	ret = e_xml_save_file (filename, doc);
+	
+	xmlFreeDoc (doc);
+	
+	return ret;
 }
 
 /**
@@ -308,10 +358,14 @@ e_table_specification_save_to_node (ETableSpecification *specification,
 	node = xmlNewNode (NULL, "ETableSpecification");
 	e_xml_set_bool_prop_by_name (node, "no-headers", specification->no_headers);
 	e_xml_set_bool_prop_by_name (node, "click-to-add", specification->click_to_add);
+	e_xml_set_bool_prop_by_name (node, "click-to-add-end", specification->click_to_add_end && specification->click_to_add);
+	e_xml_set_bool_prop_by_name (node, "alternating-row-colors", specification->alternating_row_colors);
 	e_xml_set_bool_prop_by_name (node, "horizontal-draw-grid", specification->horizontal_draw_grid);
 	e_xml_set_bool_prop_by_name (node, "vertical-draw-grid", specification->vertical_draw_grid);
 	e_xml_set_bool_prop_by_name (node, "draw-focus", specification->draw_focus);
 	e_xml_set_bool_prop_by_name (node, "horizontal-scrolling", specification->horizontal_scrolling);
+	e_xml_set_bool_prop_by_name (node, "horizontal-resize", specification->horizontal_resize);
+	e_xml_set_bool_prop_by_name (node, "allow-grouping", specification->allow_grouping);
 
 	switch (specification->selection_mode){
 	case GTK_SELECTION_SINGLE:
@@ -332,6 +386,7 @@ e_table_specification_save_to_node (ETableSpecification *specification,
 	xmlSetProp (node, "cursor-mode", s);
 
 	xmlSetProp (node, "_click-to-add-message", specification->click_to_add_message);
+	xmlSetProp (node, "gettext-domain", specification->domain);
 
 	if (specification->columns){
 		int i;
